package TreeStore;
#
# $Id: TreeStore.pm,v 1.6 2002/10/31 08:01:12 white Exp $
#
use strict;
use vars qw(@ISA $AUTOLOAD @EXPORT);
use TreeStore::Object;

my(%cache);
my($BaseDir);
my($CREATE_MODE_DIR)  = 0777;
my($CREATE_MODE_FILE) = 0666;

#sub TSTYPE_OBJECT(){ 1 }
#sub TSTYPE_PROPERTY(){ 2 }
#sub TSTYPE_UNDEF(){ 0 }

use constant OBJECT   => 1;
use constant PROPERTY => 2;
use constant UNDEF    => 0;

sub new{
    my $self  = shift;
    my $class = ref($self) || $self;
    my($base) = bless { BASE => 1 } , $class;
    $self = {path => "" , base => $base} if($class eq $self);
    my $root = $base->read(@_);
    return bless $root,"TreeStore::Object";
}

sub read{
    my($self,$path) = @_;

    if(not defined $self->{cache}->{$path}){
	$self->{cache}->{$path} = { path => $path, base => $self };
	my $child = $self->{cache}->{$path};
	if(-d $path){
	    $child->{Type} = OBJECT;
	}elsif(-f _){
	    my @read = $self->read_file($path);
	    $child->{Type}  = PROPERTY;
	    $child->{value} = \@read;
	}else{
	    $child->{Type}  = UNDEF;

	}
    }
    my $child = $self->{cache}->{$path};
    return $self->{cache}->{$path};
}

sub read_dir{
    (my $self,my $path,@_) = &__PARSEARG(@_);
    my @result = $self->read_dir_cache($path);
    if(wantarray){
        return @result;
    }else{
        return join("\n",@result);
    }
}

sub read_file{
    (my $self,my $path,@_) = &__PARSEARG(@_);
    my @result = $self->read_file_cache($path);
    if(wantarray){
        return @result;
    }else{
        return join("\n",@result);
    }
}

sub read_file_cache{
    (my $self,my $path) = @_;
    if(not defined $self->{cache}->{$path}->{value}){
	open(READ,"< $path");
	my @read = <READ>;
	close(READ);
	chomp(@read);
	$self->{cache}->{$path}->{value} = \@read;
    }
    return @{$self->{cache}->{$path}->{value}};
}

sub read_dir_cache{
    (my $self,my $path) = @_;
    if(not defined $self->{cache}->{$path}->{value}){
	opendir(DIR,"$path");
	my(@read)= readdir(DIR);
	closedir(DIR);
	
	my %tmp;
	map {$tmp{$_} = $_} @read;
	delete $tmp{"."};
	delete $tmp{".."};
	$self->{cache}->{$path}->{value} = [ keys %tmp ];
    }
    return @{$self->{cache}->{$path}->{value}};
}

sub write_file{
    (my $self,my $path,@_) = &__PARSEARG(@_);
    my($content) = shift;
    my($parent)= $path =~ /^(.*)\/.*?$/;
    if(! -e $parent){
	mkdir($parent);
    }
    open(WRITE,"> $path") or print STDERR "open error $path [$parent]\n";
    print WRITE $content;
    close(WRITE);
#    my @tmp = split(/\n/,$content);
#    $self->{cache}->{$path}->{value} = \@tmp;
}

sub join_path{
    (my $self,@_) = &__PARSEARG(@_);
    local($_);
    while(@_){
	$_ = shift(@_);
	last if($_ !~ /^\/*$/);
    }
    my $path = $_;
    foreach(@_){
	next if($_ =~ /^\/*$/);
	if($path =~ /\/$/){
	    $path .= $_;
	}else{
	    $path .= "/$_";
	}
    }
    return $path;
}

sub __PARSEARG{
    my($path) = shift;
    my($self);
    if(ref($path)){
	$self = $path;
	$path = shift;
	$path = $path->{path} if ref($path);
    }
    return ($self,$path,@_);
}

sub delete{
    my($self,$path) = @_;
    if(-d $path){
	rmdir($path) or print "rmdir $path failed\n";
    }elsif(-f $path){
	unlink($path) or print "rmdir $path failed\n"
    }
}

sub DESTROY{
# Flush out objects
    my($self) = shift;
    foreach(keys %{$self->{cache}}){
	my $child = $self->{cache}->{$_};
	if($child->{Modify} eq 'delete'){
	    if($child->{Type} == OBJECT){
		$self->delete($child->{path})
	    }elsif($child->{Type} == PROPERTY){
		$self->delete($child->{path});
	    }
	    
	}elsif($child->{Modify} eq 'write'){
	    $self->write_file($child->{path},join("\n",@{$child->{value}}));
	}
    }
    if($self->{value_seted} == 1){
	$self->flush;
    }
}
1;
